@@ -3,6 +3,7 @@
 #ifndef _I830_DRI_H
 #define _I830_DRI_H
 
+#include "xf86dri.h"
 #include "xf86drm.h"
 #include "i830_common.h"
 
@@ -10,7 +11,7 @@
 
 #define I830_MAJOR_VERSION 1
 #define I830_MINOR_VERSION 6
-#define I830_PATCHLEVEL 1
+#define I830_PATCHLEVEL 2
 
 #define I830_REG_SIZE 0x80000
 
@@ -18,21 +19,6 @@ typedef struct _I830DRIRec {
    drm_handle_t regs;
    drmSize regsSize;
 
-   drmSize backbufferSize;
-   drm_handle_t backbuffer;
-
-   drmSize depthbufferSize;
-   drm_handle_t depthbuffer;
-
-   drmSize rotatedSize;
-   drm_handle_t rotatedbuffer;
-
-   drm_handle_t textures;
-   int textureSize;
-
-   drm_handle_t agp_buffers;
-   drmSize agp_buf_size;
-
    int deviceID;
    int width;
    int height;
@@ -40,21 +26,6 @@ typedef struct _I830DRIRec {
    int cpp;
    int bitsPerPixel;
 
-   int fbOffset;
-   int fbStride;
-
-   int backOffset;
-   int backPitch;
-
-   int depthOffset;
-   int depthPitch;
-
-   int rotatedOffset;
-   int rotatedPitch;
-
-   int logTextureGranularity;
-   int textureOffset;
-
    int irq;
    int sarea_priv_offset;
 } I830DRIRec, *I830DRIPtr;
